#if defined(GET_NODE_TYPE)
NODE_TYPE_TILES,
#elif defined(GET_NODE_INFO)
	NODE_INFO_NAME("tiles   ");	// try to match 8 character
	NODE_INFO_README("\
A 2D sprite module.\n\
Draws a 2D-array(VarIn2) of tiles(Var0)\n\
on screen at pos (Var2, Var3)\n\
via CallIn0.\n\
Var1 is the number of tiles in\n\
the tileset.bmp specified by\n\
Var0.\n\
The array dimensions are linked\n\
with VarIn0 and VarIn1.\n\
Always use JumpIn0 before any\n\
usage and use JumpIn1 at exit.\n");
    NODE_INFO_FIELDS(10);
        /* a .bmp filename which contains the tileset of sprites */
    NODE_INFO_FIELD(0, FIELD_VAR, FIELD_TYPE_STRING, "tileset ");
        /* how many tiles ? */
    NODE_INFO_FIELD(1, FIELD_VAR, FIELD_TYPE_INT, "nTiles  ");
        /* where to draw? (x,y) coordinates */
    NODE_INFO_FIELD(2, FIELD_VAR, FIELD_TYPE_INT, "pos_x   ");
    NODE_INFO_FIELD(3, FIELD_VAR, FIELD_TYPE_INT, "pos_y   ");
        /* from a linked array2d: */
            /* (x,y) dimension */
    NODE_INFO_FIELD(4, FIELD_VARIN, FIELD_TYPE_INT, "dim_x   ");
    NODE_INFO_FIELD(5, FIELD_VARIN, FIELD_TYPE_INT, "dim_y   ");
            /* the array */
    NODE_INFO_FIELD(6, FIELD_VARIN, FIELD_TYPE_INT_ARRAY, "array   ");
        /* a local id to distinguish private vars not visible in editor */
    NODE_INFO_FIELD(7, FIELD_CONST, FIELD_TYPE_INT, "private ");
        /* init and exit */
    NODE_INFO_FIELD(8, FIELD_JUMPOUT, FIELD_TYPE_JUMP, "init->  ");
    NODE_INFO_FIELD(9, FIELD_JUMPOUT, FIELD_TYPE_JUMP, "exit->  ");
	NODE_INFO_JUMPINS(2);
	NODE_INFO_CALLINS(1);
	/* prepare for next module */
	NODE_INFO_NEXT;
#elif defined(GET_LABELS)
	NODE_INFO_JUMPIN(0,JUMP_IN_TILES_INIT, "->init  ");
	NODE_INFO_JUMPIN(1,JUMP_IN_TILES_EXIT, "->exit  ");
    
	NODE_INFO_CALLIN(0,CALL_IN_TILES_DRAW, "draw    ");
	/* prepare for next module */
	NODE_INFO_NEXT;
#elif defined(GET_INCLUDES)
#elif defined(GET_LOCAL_VARS)
    int tiles_n = 0;
#if defined(SDL_PC)
    SDL_Surface **tiles_set;
#endif
#elif defined(GET_GLOBAL_VARS)
#elif defined(GET_COMPUTATION)
JUMP_IN_TILES_INIT:
{
#if defined(PRINT_DEBUG)
    mprintf("module: tiles - init");
#endif

#if defined(SDL_PC)
    /* make a private var */
        /* we get this id */
    ACCESS_LOCAL_INT(7) = tiles_n;
    if(tiles_n == 0) {
        tiles_set = (SDL_Surface **)malloc(sizeof(SDL_Surface *));
        if(tiles_set == NULL) {
            mprintf("Error: no memory for private vars!");
            RETURN_ERROR;
        }
        tiles_n = 1;
    } else {
        tiles_set = (SDL_Surface **)realloc(tiles_set, (tiles_n + 1) * sizeof(SDL_Surface *));
        if(tiles_set == NULL) {
            mprintf("Error: no memory for private vars!");
            RETURN_ERROR;
        }
        tiles_n++;
    }
    /* load the tileset */
    *(tiles_set + ACCESS_LOCAL_INT(7)) = SDL_LoadBMP(ACCESS_LOCAL_STRING(0));
    if(*(tiles_set + ACCESS_LOCAL_INT(7)) == NULL) {
        mprintf("Error: could not load tileset %s!", ACCESS_LOCAL_STRING(0));
        RETURN_ERROR;
    }
    // from the SDL docs
    /*
     * Palettized screen modes will have a default palette (a standard
     * 8*8*4 colour cube), but if the image is palettized as well we can
     * use that palette for a nicer colour matching
     */
    if ((*(tiles_set + ACCESS_LOCAL_INT(7)))->format->palette && screen->format->palette) {
        SDL_SetColors(screen, (*(tiles_set + ACCESS_LOCAL_INT(7)))->format->palette->colors, 0, (*(tiles_set + ACCESS_LOCAL_INT(7)))->format->palette->ncolors);
    }
#endif

    /* test if the dimensions are correct */
    if(ACCESS_LINK_INT(4) * ACCESS_LINK_INT(5) > ACCESS_LINK_ARRAY(6)->n) {
        mprintf("Error: array does not match the dimensions!");
        RETURN_ERROR;
    }
    
    JUMP_TO_MODULE(8);
}
JUMP_IN_TILES_EXIT:
{
#if defined(PRINT_DEBUG)
    mprintf("module: tiles - exit");
#endif

#if defined(SDL_PC)
    SDL_FreeSurface(*(tiles_set + ACCESS_LOCAL_INT(7)));
#endif

    tiles_n--;
    if(tiles_n == 0) {
        /* the last one frees the private var memory */
        free(tiles_set);
    }

    JUMP_TO_MODULE(10);
}
CALL_IN_TILES_DRAW:
{
#if defined(SDL_PC)
    SDL_Rect srcrect, dstrect;
#endif
#if defined(PRINT_DEBUG)
    mprintf("module: array2d - read");
#endif

#if defined(SDL_PC)
    array = ACCESS_LINK_ARRAY(6);
    srcrect.x = 0;
    srcrect.y = 0;
    srcrect.w = (*(tiles_set + ACCESS_LOCAL_INT(7)))->w;
    srcrect.h = (*(tiles_set + ACCESS_LOCAL_INT(7)))->h / ACCESS_LOCAL_INT(1);
    dstrect.y = ACCESS_LOCAL_INT(3);
    
    for(i = 0; i < ACCESS_LINK_INT(5); i++) {
        dstrect.x = ACCESS_LOCAL_INT(2);
        for(j = 0; j < ACCESS_LINK_INT(4); j++) {
            /* select the correct tile */
            srcrect.y = *(array->field.intPtr + i * ACCESS_LINK_INT(4) + j);
                /* test bounds */
            if(srcrect.y >= ACCESS_LOCAL_INT(1)) {
                mprintf("Warning: illegal tile, default to 0!");
                srcrect.y = 0;
            }
            srcrect.y *= srcrect.h;
            /* Blit onto the screen surface */
            if(SDL_BlitSurface(*(tiles_set + ACCESS_LOCAL_INT(7)), &srcrect, screen, &dstrect) < 0)
                mprintf("Error writing tile(%i,%i): %s", j, i, SDL_GetError());

            dstrect.x += srcrect.w;
        }
        dstrect.y += srcrect.h;
    }
#endif

    JUMP_BACK;
}
#elif defined(GET_DEBUGABLE)
#endif
